/**
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 *
 * Authors:
 * 2009-
 *      Oscar Koeroo <okoeroo@nikhef.nl>
 *      Mischa Sall\'e <msalle@nikhef.nl>
 *      NIKHEF Amsterdam, the Netherlands
 *
 *  2006-2009
 *      Gerben Venekamp <venekamp@nikhef.nl>
 *      NIKHEF Amsterdam, the Netherlands
 *
 */

/*!
 *  \file glexec_ipc.h
 *
 *  Provides functionality for ipc handling: payload and epilogue, signals
 *  handling etc.
 */

#ifndef _GLEXEC_IPC_H
#define _GLEXEC_IPC_H

/* Need for sigset_t */
#include <signal.h>

/* Need for uid_t */
#include <sys/types.h>

/************************************************************************/
/* DEFINES AND TYPES                                                    */
/************************************************************************/

#ifndef GLEXEC_TERM_GRACE_TIME
#define GLEXEC_TERM_GRACE_TIME	    5
#endif

#ifndef GLEXEC_KILL_GRACE_TIME
#define GLEXEC_KILL_GRACE_TIME	    1
#endif

#ifndef GLEXEC_EPILOG_TIMEOUT
#define GLEXEC_EPILOG_TIMEOUT	    300
#endif

/************************************************************************/
/* FUNCTIONS                                                            */
/************************************************************************/

/*!
 * Sets caller id into global static.
 * \param caller_uid;
 */
void glexec_set_caller_uid(uid_t glexec_caller_uid);

/*!
 * Sets whether gLExec should call setpgid(0,0) in a child process and hence
 * send TERM and KILL signals to the process group or just the process id.
 * Default is to use setpgid()
 * \param value 1 to use setpgid 0 not to.
 */
void glexec_set_use_setpgid(int value);

/*!
 * Getter function for glexec_set_use_setpgid
 * \return 0 don't use process groups
 * \return 1 use process groups
 */
int glexec_get_use_setpgid(void);

/*
 * Sets whether gLExec will force the payload to run in the background, only
 * relevant when payload has different process group than gLExec: either payload
 * or gLExec will have the stdin tty. By default gLExec transfers to payload,
 * this flag keeps it at lingering gLExec and gives payload /dev/null */
void glexec_set_payload_should_run_in_background(int value);

/*!
 * Sets grace time after sending TERM to child process, before sending KILL.
 * Default is GLEXEC_TERM_GRACE_TIME, 0 doesn't time-out
 * \param delay in seconds
 */
void glexec_set_term_delay(unsigned delay);

/*!
 * Sets grace time after sending KILL to child process, before exiting.
 * Default is GLEXEC_KILL_GRACE_TIME, 0 doesn't time-out (don't do that!)
 * \param delay in seconds
 */
void glexec_set_kill_delay(unsigned delay);

/*!
 * Sets grace time after start of epilogue script, before sending it a SIGTERM.
 * Default is GLEXEC_EPILOG_TIMEOUT, 0 doesn't time-out (not advisable)
 * \param delay in seconds
 */
void glexec_set_epilog_timeout(unsigned timeout);

/*!
 * Sets flag to make sure epilogue will run, if already called, or child is
 * already being forked, it will be ignored.
 */
void glexec_set_epilog_should_run(void);

/*!
 * getter function for epilogue running...
 * \return 0 epilogue will not run
 * \return 1 other cases, including failed, finished etc.
 */
int glexec_get_epilog_should_run(void);

/*!
 * Sets whether gLExec should call set handlers for signals used by debuggers
 * and the like, such as SIGTRAP.
 * \param value 1 to install custom handlers, 0 not to set them.
 */
void glexec_set_use_extra_sighandlers(int value);

/*!
 * Getter function whether glexec_ipc thinks glexec can switch. This is only
 * reliable after glexec_install_sig_handlers() has been called
 * \return 1 when glexec can switch 0 otherwise
 */ 
int glexec_can_switch(void);

/*!
 * Getter function for child PID
 * \return pid_t of child process
 */
pid_t glexec_get_childpid(void);

/*!
 * Getter function for epilogue PID
 * \return pid_t of child process
 */
pid_t glexec_get_epilogpid(void);

/*!
 * Getter function for child exit code
 * \return rc of payload process
 */
int glexec_get_childrc(void);

/*!
 * Installs signal handlers and resets timer
 */
void glexec_install_sig_handlers(void);

/*!
 * Install optional extra signal handlers. We only can install these after
 * having set the flag global_ipc_extrasighandlers using the
 * glexec_set_use_extra_sighandlers setter function.
 */
void glexec_install_extra_sig_handlers(void);

/*!
 * Blocks (in addition to whatever is already blocked) all signals except for:
 * - unblockable: SIGKILL, SIGSTOP
 * - SIGABRT 
 * - core-type signals: SIGBUS, SIGFPE, SIGILL, SIGSEGV, SIGSYS
 * \param oldsigset previous signal mask
 */
void glexec_block_signals(sigset_t *old_sigset);

/*!
 * Blocks (in addition to whatever is already blocked) all signals:
 * \param oldsigset previous signal mask
 */
void glexec_block_allsignals(sigset_t *old_blocked_signals_set);

/*!
 * Set blocked signal mask to sigset, exit when fails
 * \param sigset signal set to block
 */
void glexec_sigprocmask(const sigset_t *masksigset);

/*!
 * Forks of the child process, sets the childpid and start the accounting.
 * Failure is handles internally.
 * \return 0 child
 * \return pid parent
 */
pid_t glexec_forkchild(void);

/*!
 * Forks of the epilog process, sets the epilogpid and start the accounting.
 * Failure is handles internally.
 * \return 0 epilog
 * \return pid parent
 */
pid_t glexec_forkepilog(void);

/*!
 * Waits for the child/payload process (in linger mode) to finish and correctly
 * handles, logs etc. the reason (normal exit, signal etc.). Timing statistics
 * are also logged.
 * \return exit code for child process, following (ba)sh convention: when due to
 * a signal: 128+<signal>, in case of unsignalled but abnormal program end, we
 * return 1. See http://tldp.org/LDP/abs/html/exitcodes.html for details.
 */ 
int glexec_waitchild(void);

/*!
 * Waits for the epilogue process to finish and correctly handles, logs etc. the
 * reason (normal exit, signal etc.).
 * \return exit code for epilogue process, following (ba)sh convention: when due to
 * a signal: 128+<signal>, in case of unsignalled but abnormal program end, we
 * return 1. See http://tldp.org/LDP/abs/html/exitcodes.html for details.
 */ 
int glexec_waitepilog(void);

#endif
